@@ -31,6 +31,7 @@
 
 #include "browser.h"
 #include "common.h"
+#include "conf.h"
 #include "date.h"
 #include "debug.h"
 #include "feed.h"
@@ -85,11 +86,8 @@ typedef enum {
 } open_link_target_type;
 
 static void
-launch_item_selected (open_link_target_type open_link_target)
+launch_item (itemPtr item, open_link_target_type open_link_target)
 {
-	itemPtr		item;
-
-	item = itemlist_get_selected ();
 	if (item) {
 		gchar *link = item_make_link (item);
 
@@ -114,9 +112,6 @@ launch_item_selected (open_link_target_type open_link_target)
 		} else
 			ui_show_error_box (_("This item has no link specified!"));
 
-		item_unload (item);
-	} else {
-		liferea_shell_set_important_status_bar (_("No item has been selected"));
 	}
 }
 
@@ -130,9 +125,7 @@ struct ItemListViewPrivate {
 	gboolean	batch_mode;		/*<< TRUE if we are in batch adding mode */
 	GtkTreeStore	*batch_itemstore;	/*<< GtkTreeStore prepared unattached and to be set on update() */
 
-	GtkTreeViewColumn	*enclosureColumn;
-	GtkTreeViewColumn	*faviconColumn;
-	GtkTreeViewColumn	*stateColumn;
+	GHashTable	*columns;               /*<< Named GtkTreeViewColumns */
 
 	gboolean	wideView;		/*<< TRUE if date has to be rendered into headline column (because date column is invisible) */
 };
@@ -146,6 +139,11 @@ item_list_view_finalize (GObject *object)
 {
 	ItemListViewPrivate *priv = ITEM_LIST_VIEW_GET_PRIVATE (object);
 
+	/* Disconnect the treeview signals to avoid spurious calls during teardown */
+	g_signal_handlers_disconnect_by_data (G_OBJECT (priv->treeview), object);
+
+	g_hash_table_destroy (priv->columns);
+
 	g_slist_free (priv->item_ids);
 	if (priv->batch_itemstore)
 		g_object_unref (priv->batch_itemstore);
@@ -565,7 +563,7 @@ void
 item_list_view_update (ItemListView *ilv, gboolean hasEnclosures)
 {
 	/* we depend on the fact that the third column is the enclosure icon column!!! */
-	gtk_tree_view_column_set_visible (ilv->priv->enclosureColumn, hasEnclosures);
+	gtk_tree_view_column_set_visible (g_hash_table_lookup(ilv->priv->columns, "enclosure"), hasEnclosures);
 
 	if (ilv->priv->batch_mode) {
 		item_list_view_set_tree_store (ilv, ilv->priv->batch_itemstore);
@@ -581,7 +579,7 @@ on_item_list_view_key_press_event (GtkWidget *widget, GdkEventKey *event, gpoint
 {
 	if ((event->type == GDK_KEY_PRESS) && (event->state == 0)
 	    && (event->keyval == GDK_KEY_Delete))
-		on_remove_item_activate(NULL, NULL);
+		on_action_remove_item(NULL, NULL, NULL);
 
 	return FALSE;
 }
@@ -650,6 +648,7 @@ on_item_list_view_button_press_event (GtkWidget *treeview, GdkEventButton *event
 	ItemListView		*ilv = ITEM_LIST_VIEW (user_data);
 	GtkTreePath		*path;
 	GtkTreeIter		iter;
+	GtkTreeViewColumn	*column;
 	itemPtr			item = NULL;
 	gboolean		result = FALSE;
 
@@ -660,7 +659,7 @@ on_item_list_view_button_press_event (GtkWidget *treeview, GdkEventButton *event
 	if (event->window != gtk_tree_view_get_bin_window (ilv->priv->treeview))
 		return FALSE;
 
-	if (!gtk_tree_view_get_path_at_pos (ilv->priv->treeview, (gint)event->x, (gint)event->y, &path, NULL, NULL, NULL))
+	if (!gtk_tree_view_get_path_at_pos (ilv->priv->treeview, (gint)event->x, (gint)event->y, &path, &column, NULL, NULL))
 		return FALSE;
 
 	if (gtk_tree_model_get_iter (gtk_tree_view_get_model (ilv->priv->treeview), &iter, path))
@@ -672,21 +671,10 @@ on_item_list_view_button_press_event (GtkWidget *treeview, GdkEventButton *event
 		GdkEventButton *eb = (GdkEventButton*)event;
 		switch (eb->button) {
 			case 1:
-				/* Allow flag toggling when left clicking in the
-				   state or favicon column. We depend
-				   on the fact that those columns are all left
-				   of the headline column !!! */
-				if(gtk_tree_view_column_get_visible (ilv->priv->faviconColumn)){
-					if (event->x <= (gtk_tree_view_column_get_width (ilv->priv->faviconColumn))) {
-						itemlist_toggle_flag (item);
-						result = TRUE;
-					}
-				}
-				else{
-					if (event->x <= (gtk_tree_view_column_get_width (ilv->priv->stateColumn))) {
-						itemlist_toggle_flag (item);
-						result = TRUE;
-					}
+				if (column == g_hash_table_lookup(ilv->priv->columns, "favicon") ||
+				    column == g_hash_table_lookup(ilv->priv->columns, "state")) {
+					itemlist_toggle_flag (item);
+					result = TRUE;
 				}
 				break;
 			case 2:
@@ -695,7 +683,6 @@ on_item_list_view_button_press_event (GtkWidget *treeview, GdkEventButton *event
 				result = TRUE;
 				break;
 			case 3:
-				item_list_view_select (ilv, item);
 				ui_popup_item_menu (item, eb->button, eb->time);
 				result = TRUE;
 				break;
@@ -727,12 +714,50 @@ on_item_list_view_popup_menu (GtkWidget *widget, gpointer user_data)
 }
 
 static void
-on_Itemlist_row_activated (GtkTreeView *treeview,
+on_item_list_row_activated (GtkTreeView *treeview,
                            GtkTreePath *path,
 			   GtkTreeViewColumn *column,
 			   gpointer user_data)
 {
-    launch_item_selected (DEFAULT);
+	GtkTreeModel *model = gtk_tree_view_get_model (treeview);
+	GtkTreeIter iter;
+
+	if (gtk_tree_model_get_iter (model, &iter, path)) {
+		itemPtr item = item_load (item_list_view_iter_to_id (ITEM_LIST_VIEW (user_data), &iter));
+		launch_item (item, DEFAULT);
+		item_unload (item);
+	}
+}
+
+static void
+on_item_list_view_columns_changed (GtkTreeView *treeview, ItemListView *ilv)
+{
+	gint i = 0;
+	GList *columns;
+	GHashTableIter iter;
+	gpointer colname, colptr;
+	gchar *strv[6];
+
+	/* This handler is only used for drag and drop reordering, so it
+	   should not be hooked up with less than the full number of columns
+	   eg: on item_list_view creation or teardown */
+	if (gtk_tree_view_get_n_columns(treeview) != 5)
+		return;
+
+	columns = gtk_tree_view_get_columns (treeview);
+	for (GList *li = columns; li; li = li->next) {
+		g_hash_table_iter_init (&iter, ilv->priv->columns);
+		while (g_hash_table_iter_next (&iter, &colname, &colptr)) {
+			if (li->data == colptr) {
+				strv[i++] = colname;
+				strv[i] = NULL;
+				break;
+			}
+		}
+	}
+	conf_set_strv_value (LIST_VIEW_COLUMN_ORDER, strv);
+
+	g_list_free (columns);
 }
 
 GtkWidget *
@@ -765,10 +790,13 @@ item_list_view_create (gboolean wide)
 	ItemListView		*ilv;
 	GtkCellRenderer		*renderer;
 	GtkTreeViewColumn 	*column, *headline_column;
+	gchar			**conf_column_order;
 
 	ilv = g_object_new (ITEM_LIST_VIEW_TYPE, NULL);
 	ilv->priv->wideView = wide;
-		
+
+	ilv->priv->columns = g_hash_table_new (g_str_hash, g_str_equal);
+
 	ilv->priv->ilscrolledwindow = gtk_scrolled_window_new (NULL, NULL);
 	g_object_ref_sink (ilv->priv->ilscrolledwindow);
 	gtk_widget_show (ilv->priv->ilscrolledwindow);
@@ -793,8 +821,7 @@ item_list_view_create (gboolean wide)
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	column = gtk_tree_view_column_new_with_attributes ("", renderer, "gicon", IS_STATEICON, NULL);
 	g_object_set (renderer, "stock-size", wide?GTK_ICON_SIZE_LARGE_TOOLBAR:GTK_ICON_SIZE_SMALL_TOOLBAR, NULL);
-	gtk_tree_view_append_column (ilv->priv->treeview, column);
-	ilv->priv->stateColumn = column;
+	g_hash_table_insert (ilv->priv->columns, "state", column);
 	gtk_tree_view_column_set_sort_column_id (column, IS_STATE);
 	if (wide)
 		gtk_tree_view_column_set_visible (column, FALSE);
@@ -804,9 +831,7 @@ item_list_view_create (gboolean wide)
 	g_object_set (renderer, "stock-size", wide?GTK_ICON_SIZE_DIALOG:GTK_ICON_SIZE_SMALL_TOOLBAR, NULL);
 
 	gtk_tree_view_column_set_sort_column_id (column, IS_SOURCE);
-	gtk_tree_view_append_column (ilv->priv->treeview, column);
-	ilv->priv->faviconColumn = column;
-
+	g_hash_table_insert (ilv->priv->columns, "favicon", column);
 
 	renderer = gtk_cell_renderer_text_new ();
 	headline_column = gtk_tree_view_column_new_with_attributes (_("Headline"), renderer,
@@ -814,7 +839,7 @@ item_list_view_create (gboolean wide)
 							   "xalign", ITEMSTORE_ALIGN,
 							   NULL);
 	gtk_tree_view_column_set_expand (headline_column, TRUE);
-	gtk_tree_view_append_column (ilv->priv->treeview, headline_column);
+	g_hash_table_insert (ilv->priv->columns, "headline", headline_column);
 	g_object_set (headline_column, "resizable", TRUE, NULL);
 	if (wide) {
 		gtk_tree_view_column_set_sort_column_id (headline_column, IS_TIME);
@@ -828,8 +853,7 @@ item_list_view_create (gboolean wide)
 
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	column = gtk_tree_view_column_new_with_attributes ("", renderer, "gicon", IS_ENCICON, NULL);
-	gtk_tree_view_append_column (ilv->priv->treeview, column);
-	ilv->priv->enclosureColumn = column;
+	g_hash_table_insert (ilv->priv->columns, "enclosure", column);
 
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (_("Date"), renderer,
@@ -837,14 +861,23 @@ item_list_view_create (gboolean wide)
 	                                                   "weight", ITEMSTORE_WEIGHT,
 							   NULL);
 	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);
-	gtk_tree_view_append_column (ilv->priv->treeview, column);
+	g_hash_table_insert (ilv->priv->columns, "date", column);
 	gtk_tree_view_column_set_sort_column_id(column, IS_TIME);
 	if (wide)
 		gtk_tree_view_column_set_visible (column, FALSE);
 
+	conf_get_strv_value (LIST_VIEW_COLUMN_ORDER, &conf_column_order);
+	for (gchar **li = conf_column_order; *li; li++) {
+		column = g_hash_table_lookup (ilv->priv->columns, *li);
+		g_object_set (column, "reorderable", TRUE, NULL);
+		gtk_tree_view_append_column (ilv->priv->treeview, column);
+	}
+	g_strfreev (conf_column_order);
+
 	/* And connect signals */
 	g_signal_connect (G_OBJECT (ilv->priv->treeview), "button_press_event", G_CALLBACK (on_item_list_view_button_press_event), ilv);
-	g_signal_connect (G_OBJECT (ilv->priv->treeview), "row_activated", G_CALLBACK (on_Itemlist_row_activated), ilv);
+	g_signal_connect (G_OBJECT (ilv->priv->treeview), "columns_changed", G_CALLBACK (on_item_list_view_columns_changed), ilv);
+	g_signal_connect (G_OBJECT (ilv->priv->treeview), "row_activated", G_CALLBACK (on_item_list_row_activated), ilv);
 	g_signal_connect (G_OBJECT (ilv->priv->treeview), "key-press-event", G_CALLBACK (on_item_list_view_key_press_event), ilv);
 	g_signal_connect (G_OBJECT (ilv->priv->treeview), "popup_menu", G_CALLBACK (on_item_list_view_popup_menu), ilv);
 
@@ -861,8 +894,7 @@ item_list_view_add_item_to_tree_store (ItemListView *ilv, GtkTreeStore *itemstor
 {
 	gint		state = 0;
 	nodePtr		node;
-	GtkTreeIter	*iter;
-	GtkTreeIter	old_iter;
+	GtkTreeIter	iter;
 	gboolean	exists;
 
 	if (item->flagStatus)
@@ -874,17 +906,15 @@ item_list_view_add_item_to_tree_store (ItemListView *ilv, GtkTreeStore *itemstor
 	if(!node)
 		return;	/* comment items do cause this... maybe filtering them earlier would be a good idea... */
 
-	exists = item_list_view_id_to_iter (ilv, item->id, &old_iter);
-	iter = &old_iter;
+	exists = item_list_view_id_to_iter (ilv, item->id, &iter);
 
 	if (!exists)
 	{
-		iter = g_new0 (GtkTreeIter, 1);
-		gtk_tree_store_prepend (itemstore, iter, NULL);
+		gtk_tree_store_prepend (itemstore, &iter, NULL);
 		ilv->priv->item_ids = g_slist_append (ilv->priv->item_ids, GUINT_TO_POINTER (item->id));
 	}
 
-	gtk_tree_store_set (itemstore, iter,
+	gtk_tree_store_set (itemstore, &iter,
 		                       IS_TIME, item->time,
 		                       IS_NR, item->id,
 				       IS_PARENT, node,
@@ -916,39 +946,69 @@ item_list_view_add_item (ItemListView *ilv, itemPtr item)
 void
 item_list_view_enable_favicon_column (ItemListView *ilv, gboolean enabled)
 {
-	gtk_tree_view_column_set_visible (ilv->priv->faviconColumn, enabled);
+	gtk_tree_view_column_set_visible (g_hash_table_lookup(ilv->priv->columns, "favicon"), enabled);
 
 	// In wide view we want to save vertical space and hide the state column
 	if (ilv->priv->wideView)
-		gtk_tree_view_column_set_visible (ilv->priv->stateColumn, !enabled);
+		gtk_tree_view_column_set_visible (g_hash_table_lookup(ilv->priv->columns, "state"), !enabled);
 }
 
 void
-on_popup_launch_item_selected (void)
+on_action_launch_item_in_browser (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-	launch_item_selected (INTERNAL);
+	itemPtr item = NULL;
+	if (parameter)
+		item = item_load (g_variant_get_uint64 (parameter));
+	else
+		item = itemlist_get_selected ();
+
+	if (item) {
+	      launch_item (item, INTERNAL);
+	      item_unload (item);
+	}
 }
 
 void
-on_popup_launch_item_in_tab_selected (void)
+on_action_launch_item_in_tab (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-	launch_item_selected (TAB);
+	itemPtr item = NULL;
+	if (parameter)
+		item = item_load (g_variant_get_uint64 (parameter));
+	else
+		item = itemlist_get_selected ();
+
+	if (item) {
+	      launch_item (item, TAB);
+	      item_unload (item);
+	}
 }
 
 void
-on_popup_launch_item_external_selected (void)
+on_action_launch_item_in_external_browser (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-	launch_item_selected (EXTERNAL);
+	itemPtr item = NULL;
+	if (parameter)
+		item = item_load (g_variant_get_uint64 (parameter));
+	else
+		item = itemlist_get_selected ();
+
+	if (item) {
+	      launch_item (item, EXTERNAL);
+	      item_unload (item);
+	}
 }
 
 /* menu callbacks */
 
 void
-on_toggle_item_flag (GtkMenuItem *menuitem, gpointer user_data)
+on_toggle_item_flag (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-	itemPtr		item;
+	itemPtr item = NULL;
+	if (parameter)
+		item = item_load (g_variant_get_uint64 (parameter));
+	else
+		item = itemlist_get_selected ();
 
-	item = itemlist_get_selected ();
 	if (item) {
 		itemlist_toggle_flag (item);
 		item_unload (item);
@@ -956,17 +1016,14 @@ on_toggle_item_flag (GtkMenuItem *menuitem, gpointer user_data)
 }
 
 void
-on_popup_toggle_flag (void)
+on_toggle_unread_status (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-	on_toggle_item_flag (NULL, NULL);
-}
+	itemPtr item = NULL;
+	if (parameter)
+		item = item_load (g_variant_get_uint64 (parameter));
+	else
+		item = itemlist_get_selected ();
 
-void
-on_toggle_unread_status (GtkMenuItem *menuitem, gpointer user_data)
-{
-	itemPtr		item;
-
-	item = itemlist_get_selected ();
 	if (item) {
 		itemlist_toggle_read_status (item);
 		item_unload (item);
@@ -974,13 +1031,7 @@ on_toggle_unread_status (GtkMenuItem *menuitem, gpointer user_data)
 }
 
 void
-on_popup_toggle_read (void)
-{
-	on_toggle_unread_status (NULL, NULL);
-}
-
-void
-on_remove_items_activate (GtkMenuItem *menuitem, gpointer user_data)
+on_remove_items_activate (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
 	nodePtr		node;
 
@@ -993,11 +1044,14 @@ on_remove_items_activate (GtkMenuItem *menuitem, gpointer user_data)
 }
 
 void
-on_remove_item_activate (GtkMenuItem *menuitem, gpointer user_data)
+on_action_remove_item (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-	itemPtr		item;
+	itemPtr item = NULL;
+	if (parameter)
+		item = item_load (g_variant_get_uint64 (parameter));
+	else
+		item = itemlist_get_selected ();
 
-	item = itemlist_get_selected ();
 	if (item) {
 		itemview_select_item (NULL);
 		itemlist_remove_item (item);
@@ -1006,12 +1060,6 @@ on_remove_item_activate (GtkMenuItem *menuitem, gpointer user_data)
 	}
 }
 
-void
-on_popup_remove_selected (void)
-{
-	on_remove_item_activate (NULL, NULL);
-}
-
 void
 item_list_view_select (ItemListView *ilv, itemPtr item)
 {
@@ -1065,13 +1113,13 @@ item_list_view_find_unread_item (ItemListView *ilv, gulong startId)
 }
 
 void
-on_next_unread_item_activate (GtkMenuItem *menuitem, gpointer user_data)
+on_next_unread_item_activate (GSimpleAction *menuitem, GVariant*parameter, gpointer user_data)
 {
 	itemlist_select_next_unread ();
 }
 
 void
-on_popup_copy_URL_clipboard (void)
+on_popup_copy_URL_clipboard (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
 	itemPtr         item;
 
@@ -1090,7 +1138,7 @@ on_popup_copy_URL_clipboard (void)
 }
 
 void
-on_popup_social_bm_item_selected (void)
+on_popup_social_bm_item_selected (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
 	itemPtr item;
 
